/*!
 * @visao/viewer-api v0.0.1
 * (c) Visao
 * Released under the MIT License.
 */

'use strict';

var _a;
/**
 * Represents the state of the viewer. Certain methods will need to have a or multiple specific status on order
 * to be executed.
 *
 * > `unmounted` -> viewer's HTML element is yet to be mounted in the DOM.
 *
 * > `mounted` -> viewer's HTML element has been mounted in the DOM.
 *
 * > `loading` -> a 3D model is currently loading into the viewer.
 *
 * > `loaded` -> a 3D is loaded in the viewer and is ready to be manipulated.
 */
exports.ViewerStatus = void 0;
(function (ViewerStatus) {
    ViewerStatus["UNMOUNTED"] = "unmounted";
    ViewerStatus["MOUNTED"] = "mounted";
    ViewerStatus["LOADING"] = "loading";
    ViewerStatus["LOADED"] = "loaded";
})(exports.ViewerStatus || (exports.ViewerStatus = {}));
/**
 * @ignore
 */
var statusLevels = (_a = {},
    _a[exports.ViewerStatus.UNMOUNTED] = 1,
    _a[exports.ViewerStatus.MOUNTED] = 2,
    _a[exports.ViewerStatus.LOADING] = 3,
    _a[exports.ViewerStatus.LOADED] = 4,
    _a);
/**
 * @ignore
 */
exports.ViewerAPIMessage = void 0;
(function (ViewerAPIMessage) {
    // GETTERS - Request the viewer
    ViewerAPIMessage["GET_VIEWER"] = "VISAO/GET_VIEWER";
    ViewerAPIMessage["GET_STATUS"] = "VISAO/GET_STATUS";
    ViewerAPIMessage["GET_VARIANT_INFO"] = "VISAO/GET_VARIANT_INFO";
    ViewerAPIMessage["GET_LANGUAGE_INFO"] = "VISAO/GET_LANGUAGE_INFO";
    ViewerAPIMessage["GET_ANIMATION_INFO"] = "VISAO/GET_ANIMATION_INFO";
    ViewerAPIMessage["GET_DIMENSIONS_UNIT"] = "VISAO/GET_DIMENSIONS_UNIT";
    ViewerAPIMessage["GET_DEMOS"] = "VISAO/GET_DEMOS";
    ViewerAPIMessage["GET_SCREENSHOT"] = "VISAO/GET_SCREENSHOT";
    // SENDERS - Send back to the client
    ViewerAPIMessage["SEND_VIEWER"] = "VISAO/SEND_VIEWER";
    ViewerAPIMessage["SEND_STATUS"] = "VISAO/SEND_STATUS";
    ViewerAPIMessage["SEND_VARIANT_INFO"] = "VISAO/SEND_VARIANT_INFO";
    ViewerAPIMessage["SEND_LANGUAGE_INFO"] = "VISAO/SEND_LANGUAGE_INFO";
    ViewerAPIMessage["SEND_ANIMATION_INFO"] = "VISAO/SEND_ANIMATION_INFO";
    ViewerAPIMessage["SEND_DIMENSIONS_UNIT"] = "VISAO/SEND_DIMENSIONS_UNIT";
    ViewerAPIMessage["SEND_DEMOS"] = "VISAO/SEND_DEMOS";
    ViewerAPIMessage["SEND_SCREENSHOT"] = "VISAO/SEND_SCREENSHOT";
    // ACTIONS - Execute in the viewer
    ViewerAPIMessage["UPDATE_VARIANT"] = "VISAO/UPDATE_VARIANT";
    ViewerAPIMessage["REMOVE_VARIANT"] = "VISAO/REMOVE_VARIANT";
    ViewerAPIMessage["UPDATE_LANGUAGE"] = "VISAO/UPDATE_LANGUAGE";
    ViewerAPIMessage["UPDATE_DIMENSIONS_UNIT"] = "VISAO/UPDATE_DIMENSIONS_UNIT";
    ViewerAPIMessage["START_AR"] = "VISAO/START_AR";
    ViewerAPIMessage["SHOW_STEP"] = "VISAO/SHOW_STEP";
    ViewerAPIMessage["NEXT_STEP"] = "VISAO/NEXT_STEP";
    ViewerAPIMessage["PREVIOUS_STEP"] = "VISAO/PREVIOUS_STEP";
    ViewerAPIMessage["CLOSE_STEP"] = "VISAO/CLOSE_STEP";
    ViewerAPIMessage["PLAY"] = "VISAO/PLAY";
    ViewerAPIMessage["PAUSE"] = "VISAO/PAUSE";
    ViewerAPIMessage["CAPTURE_SCREENSHOT"] = "VISAO/CAPTURE_SCREENSHOT";
    ViewerAPIMessage["RESET_CAMERA"] = "VISAO/RESET_CAMERA";
    ViewerAPIMessage["LOCK_CAMERA"] = "VISAO/LOCK_CAMERA";
    ViewerAPIMessage["UNLOCK_CAMERA"] = "VISAO/UNLOCK_CAMERA";
    ViewerAPIMessage["SHOW_HELP"] = "VISAO/SHOW_HELP";
    ViewerAPIMessage["CLOSE_HELP"] = "VISAO/CLOSE_HELP";
    ViewerAPIMessage["SHOW_QR"] = "VISAO/SHOW_QR";
    ViewerAPIMessage["CLOSE_QR"] = "VISAO/CLOSE_QR";
    ViewerAPIMessage["SHOW_DIMENSIONS"] = "VISAO/SHOW_DIMENSIONS";
    ViewerAPIMessage["CLOSE_DIMENSIONS"] = "VISAO/CLOSE_DIMENSIONS";
    ViewerAPIMessage["START_AUTO_ROTATE"] = "VISAO/START_AUTO_ROTATE";
    ViewerAPIMessage["STOP_AUTO_ROTATE"] = "VISAO/STOP_AUTO_ROTATE";
    ViewerAPIMessage["MUTE_VOICEOVER"] = "VISAO/MUTE_VOICEOVER";
    ViewerAPIMessage["UNMUTE_VOICEOVER"] = "VISAO/UNMUTE_VOICEOVER";
    ViewerAPIMessage["START_ANIMATION"] = "VISAO/START_ANIMATION";
    ViewerAPIMessage["STOP_ANIMATION"] = "VISAO/STOP_ANIMATION";
    // LEGACY
    ViewerAPIMessage["STATUS"] = "VISAO/STATUS";
    ViewerAPIMessage["GET_VARIANT_INFORMATION"] = "VISAO/GET_VARIANT_INFORMATION";
    ViewerAPIMessage["GET_LANGUAGE_INFORMATION"] = "VISAO/GET_LANGUAGE_INFORMATION";
})(exports.ViewerAPIMessage || (exports.ViewerAPIMessage = {}));
/**
 * @ignore
 */
exports.ViewerAPIMessageSource = void 0;
(function (ViewerAPIMessageSource) {
    ViewerAPIMessageSource["VIEWER"] = "visao_viewer";
    ViewerAPIMessageSource["API"] = "visao_viewer_api";
})(exports.ViewerAPIMessageSource || (exports.ViewerAPIMessageSource = {}));
exports.AnimationRepetitionType = void 0;
(function (AnimationRepetitionType) {
    AnimationRepetitionType["LOOP"] = "loop";
    AnimationRepetitionType["REPEAT"] = "repeat";
})(exports.AnimationRepetitionType || (exports.AnimationRepetitionType = {}));
var defaultScreenshotOptions = {
    mimeType: 'image/png',
    fileFormat: 'blob',
};

/**
 * @ignore
 */
var listenToMessages = function (incomingMessageHandler) {
    window.addEventListener('message', incomingMessageHandler);
};
/**
 * @ignore
 */
var unListenToMessages = function (incomingMessageHandler) {
    window.removeEventListener('message', incomingMessageHandler);
};
/**
 * @ignore
 */
var parseIncomingMessage = function (event, source) {
    try {
        if (!event.data || typeof event.data !== 'string') {
            return undefined;
        }
        var message = JSON.parse(event.data);
        if ((message === null || message === void 0 ? void 0 : message.source) !== source) {
            return undefined;
        }
        return message;
    }
    catch (error) {
        console.error("".concat(source, " - Could not parse the latest message"), event, error);
        return undefined;
    }
};

var LocalizedContent = /** @class */ (function () {
    function LocalizedContent(props) {
        if (props === void 0) { props = {}; }
        this.language = 'en';
        this.content = '';
        Object.assign(this, props);
    }
    return LocalizedContent;
}());

var Step = /** @class */ (function () {
    function Step(props) {
        if (props === void 0) { props = {}; }
        this.id = '';
        this.name = '';
        this.position = 0;
        this.duration = 0;
        this.localizations = [];
        Object.assign(this, props);
    }
    return Step;
}());

var Demo = /** @class */ (function () {
    function Demo(props) {
        if (props === void 0) { props = {}; }
        this.id = '';
        this.name = '';
        this.steps = [];
        this.localizations = [];
        Object.assign(this, props);
    }
    return Demo;
}());

/******************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */
/* global Reflect, Promise, SuppressedError, Symbol */


var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

function __spreadArray(to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
}

typeof SuppressedError === "function" ? SuppressedError : function (error, suppressed, message) {
    var e = new Error(message);
    return e.name = "SuppressedError", e.error = error, e.suppressed = suppressed, e;
};

function logErrorBlock(messageBlock) {
    if (messageBlock === void 0) { messageBlock = []; }
    var header = 'VISAO > An error occurred while using the viewer API:';
    console.error(__spreadArray([header], messageBlock, true).join('\n\t\t'));
}

/**
 * @ignore
 */
var Callbacks = /** @class */ (function () {
    function Callbacks(store) {
        this.store = store;
    }
    Callbacks.prototype.getFrom = function (key) {
        var _a;
        return (_a = this.store[key]) !== null && _a !== void 0 ? _a : [];
    };
    Callbacks.prototype.addTo = function (key, callbackFn) {
        var callback = { id: Date.now().toString(), fn: callbackFn };
        this.store[key] = __spreadArray(__spreadArray([], this.getFrom(key), true), [callback], false);
        return callback.id;
    };
    Callbacks.prototype.removeByCallbackFunctionFrom = function (key, callbackFn) {
        this.store[key] = this.getFrom(key).filter(function (callback) { return callback.fn !== callbackFn; });
    };
    Callbacks.prototype.removeByCallbackIdFrom = function (key, callbackId) {
        if (!callbackId) {
            return;
        }
        this.store[key] = this.getFrom(key).filter(function (callback) { return callback.id !== callbackId; });
    };
    Callbacks.prototype.executeFor = function (key, payload) {
        this.getFrom(key).forEach(function (callback) {
            callback.fn(payload);
        });
    };
    return Callbacks;
}());

/**
 * @ignore
 */
var base64ToBlob = function (base64) {
    var base64Parts = base64.split(',');
    if (base64Parts.length < 2) {
        throw new Error('Invalid base64 string');
    }
    // Decode base64 to binary string
    var byteString = atob(base64Parts[1]);
    // Extract the mime type from the base64 header
    var mimeString = base64Parts[0].split(':')[1].split(';')[0];
    // Create a Uint8Array from the binary string
    var byteArray = new Uint8Array(byteString.length);
    for (var i = 0; i < byteString.length; i++) {
        byteArray[i] = byteString.charCodeAt(i);
    }
    // Return the Blob object
    return new Blob([byteArray], { type: mimeString });
};

/**
 * The Visao class is the starting point of the Visao viewer api.
 * It encapsulates all the necessary calls to interact with your viewers from your
 * own code base.
 */
var Visao = /** @class */ (function () {
    /**
     * #### Description
     * The constructor requires an element id given to the iframe tag/component
     * defined in your application. Internally, we take care of finding the element
     * in the DOM once the component is mounted.
     *
     * #### Usage
     *
     * ```typescript
     * const instance = new Visao("viewer-iframe-id");
     * ```
     *
     * @param id
     * The id of the viewer element.
     */
    function Visao(id) {
        var _a;
        var _this = this;
        this.viewerElementInternal = null;
        this.status = exports.ViewerStatus.UNMOUNTED;
        this.handleIncomingMessage = function (event) {
            var message = parseIncomingMessage(event, exports.ViewerAPIMessageSource.VIEWER);
            if (!message) {
                return;
            }
            switch (message.type) {
                case exports.ViewerAPIMessage.SEND_STATUS: {
                    var status_1 = message.payload.status;
                    _this.status = status_1;
                    if (status_1 === exports.ViewerStatus.MOUNTED) {
                        _this.setViewerElement(document.getElementById(_this.id));
                    }
                    _this.callbacks.executeFor(exports.ViewerAPIMessage.GET_STATUS, status_1);
                    break;
                }
                case exports.ViewerAPIMessage.SEND_LANGUAGE_INFO: {
                    var languageInformation = message.payload;
                    _this.callbacks.executeFor(exports.ViewerAPIMessage.GET_LANGUAGE_INFO, languageInformation);
                    _this.callbacks.removeByCallbackIdFrom(exports.ViewerAPIMessage.GET_LANGUAGE_INFO, message.callbackId);
                    break;
                }
                case exports.ViewerAPIMessage.SEND_VARIANT_INFO: {
                    var variantInformation = message.payload;
                    _this.callbacks.executeFor(exports.ViewerAPIMessage.GET_VARIANT_INFO, variantInformation);
                    _this.callbacks.removeByCallbackIdFrom(exports.ViewerAPIMessage.GET_VARIANT_INFO, message.callbackId);
                    break;
                }
                case exports.ViewerAPIMessage.SEND_ANIMATION_INFO: {
                    var animationInformation = message.payload;
                    _this.callbacks.executeFor(exports.ViewerAPIMessage.GET_ANIMATION_INFO, animationInformation);
                    _this.callbacks.removeByCallbackIdFrom(exports.ViewerAPIMessage.GET_ANIMATION_INFO, message.callbackId);
                    break;
                }
                case exports.ViewerAPIMessage.SEND_DIMENSIONS_UNIT: {
                    var dimensionsUnitPayload = message.payload;
                    _this.callbacks.executeFor(exports.ViewerAPIMessage.GET_DIMENSIONS_UNIT, dimensionsUnitPayload);
                    _this.callbacks.removeByCallbackIdFrom(exports.ViewerAPIMessage.GET_DIMENSIONS_UNIT, message.callbackId);
                    break;
                }
                case exports.ViewerAPIMessage.SEND_SCREENSHOT: {
                    var _a = message.payload, screenshot = _a.screenshot, fileFormat = _a.options.fileFormat;
                    try {
                        var formattedScreenshot = fileFormat === 'blob' ? base64ToBlob(screenshot) : screenshot;
                        _this.callbacks.executeFor(exports.ViewerAPIMessage.GET_SCREENSHOT, formattedScreenshot);
                    }
                    catch (e) {
                        logErrorBlock(['Unable to capture the screenshot image']);
                    }
                    _this.callbacks.removeByCallbackIdFrom(exports.ViewerAPIMessage.GET_SCREENSHOT, message.callbackId);
                    break;
                }
                case exports.ViewerAPIMessage.SEND_DEMOS: {
                    var demos = message.payload;
                    _this.callbacks.executeFor(exports.ViewerAPIMessage.GET_DEMOS, demos);
                    _this.callbacks.removeByCallbackIdFrom(exports.ViewerAPIMessage.GET_DEMOS, message.callbackId);
                    break;
                }
            }
        };
        this.id = id;
        this.callbacks = new Callbacks((_a = {},
            _a[exports.ViewerAPIMessage.GET_STATUS] = [],
            _a[exports.ViewerAPIMessage.GET_LANGUAGE_INFO] = [],
            _a[exports.ViewerAPIMessage.GET_VARIANT_INFO] = [],
            _a[exports.ViewerAPIMessage.GET_ANIMATION_INFO] = [],
            _a[exports.ViewerAPIMessage.GET_DEMOS] = [],
            _a[exports.ViewerAPIMessage.GET_DIMENSIONS_UNIT] = [],
            _a[exports.ViewerAPIMessage.GET_SCREENSHOT] = [],
            _a));
        listenToMessages(this.handleIncomingMessage);
    }
    Object.defineProperty(Visao.prototype, "viewerElement", {
        get: function () {
            if (this.viewerElementInternal) {
                return this.viewerElementInternal;
            }
            if (this.id) {
                this.viewerElementInternal = document.getElementById(this.id);
                return this.viewerElementInternal;
            }
            return null;
        },
        enumerable: false,
        configurable: true
    });
    /**
     * #### Description
     * > Manually set the reference of the viewer from the id.
     *
     * #### Usage:
     *
     * ```typescript
     * visao.setViewerElementFromId('viewer-id');
     * ```
     *
     *  @param id
     *  HTML element id representing the viewer
     */
    Visao.prototype.setViewerElementFromId = function (id) {
        this.id = id;
        this.viewerElementInternal = document.getElementById(id);
    };
    /**
     * #### Description
     * > Manually set the reference of the viewer element.
     *
     * #### Usage:
     *
     * ```typescript
     * const element = document.getElementById('viewer-id');
     * visao.setViewerElement(element);
     * ```
     *
     *  @param viewerElement
     *  HTML element representing the viewer
     */
    Visao.prototype.setViewerElement = function (viewerElement) {
        this.viewerElementInternal = viewerElement;
    };
    /**
     * #### Description
     * > Listens to the state of the viewer. It will notify by calling
     * the provided parameter callback providing the new state everytime
     * the state changes.
     *
     * #### Usage
     *
     * ```typescript
     * visao.listenToViewerStatus((newViewerStatus: ViewerStatus) => {
     *   // Do Something
     * });
     * ```
     *
     * @param callback
     * Callback to execute when a change happen
     */
    Visao.prototype.listenToViewerStatus = function (callback) {
        this.callbacks.addTo(exports.ViewerAPIMessage.GET_STATUS, callback);
    };
    /**
     * #### Description
     * > Removes a viewer status listener.
     *
     * #### Usage
     *
     * ```typescript
     * visao.unListenToViewerStatus(callback);
     * ```
     *
     * @param callback
     * Callback which matches the one provided to a previously called {@link listenToViewerStatus}
     */
    Visao.prototype.unListenToViewerStatus = function (callback) {
        this.callbacks.removeByCallbackFunctionFrom(exports.ViewerAPIMessage.GET_STATUS, callback);
    };
    /**
     * #### Description
     * > Listens to any change to variants in the viewer. It will notify by calling
     * the provided parameter callback providing the new state everytime
     * the state changes.
     *
     * #### Usage
     *
     * ```typescript
     * visao.listenToVariantInfoChange((variantInformation: VariantInformationPayload) => {
     *   // Do Something
     * });
     * ```
     *
     * @param callback
     * Callback to execute when a change happen
     */
    Visao.prototype.listenToVariantInfoChange = function (callback) {
        this.callbacks.addTo(exports.ViewerAPIMessage.GET_VARIANT_INFO, callback);
    };
    /**
     * #### Description
     * > Removes a variant info change listener.
     *
     * #### Usage
     *
     * ```typescript
     * visao.unListenToVariantInfoChange(callback);
     * ```
     *
     * @param callback
     * Callback which matches the one provided to a previously called {@link listenToVariantInfoChange}
     */
    Visao.prototype.unListenToVariantInfoChange = function (callback) {
        this.callbacks.removeByCallbackFunctionFrom(exports.ViewerAPIMessage.GET_VARIANT_INFO, callback);
    };
    /**
     * #### Description
     * > Listens to any change to animations in the viewer. It will notify by calling
     * the provided parameter callback providing the new state everytime
     * the state changes.
     *
     * #### Usage
     *
     * ```typescript
     * visao.listenToAnimationInfoChange((animationInformation: AnimationInformationPayload) => {
     *   // Do Something
     * });
     * ```
     *
     * @param callback
     * Callback to execute when a change happen
     */
    Visao.prototype.listenToAnimationInfoChange = function (callback) {
        this.callbacks.addTo(exports.ViewerAPIMessage.GET_ANIMATION_INFO, callback);
    };
    /**
     * #### Description
     * > Removes an animation info change listener.
     *
     * #### Usage
     *
     * ```typescript
     * visao.unListenToAnimationInfoChange(callback);
     * ```
     *
     * @param callback
     * Callback which matches the one provided to a previously called {@link listenToAnimationInfoChange}
     */
    Visao.prototype.unListenToAnimationInfoChange = function (callback) {
        this.callbacks.removeByCallbackFunctionFrom(exports.ViewerAPIMessage.GET_ANIMATION_INFO, callback);
    };
    /**
     * #### Description
     * > Listens to any change to the language in the viewer. It will notify by calling
     * the provided parameter callback providing the new state everytime
     * the state changes.
     *
     * #### Usage
     *
     * ```typescript
     * visao.listenToLanguageInfoChange((languageInformation: LanguageInformationPayload) => {
     *   // Do Something
     * });
     * ```
     *
     * @param callback
     * Callback to execute when a change happen
     */
    Visao.prototype.listenToLanguageInfoChange = function (callback) {
        this.callbacks.addTo(exports.ViewerAPIMessage.GET_LANGUAGE_INFO, callback);
    };
    /**
     * #### Description
     * > Removes a language info change listener.
     *
     * #### Usage
     *
     * ```typescript
     * visao.unListenToLanguageInfoChange(callback);
     * ```
     *
     * @param callback
     * Callback which matches the one provided to a previously called {@link listenToLanguageInfoChange}
     */
    Visao.prototype.unListenToLanguageInfoChange = function (callback) {
        this.callbacks.removeByCallbackFunctionFrom(exports.ViewerAPIMessage.GET_LANGUAGE_INFO, callback);
    };
    /**
     * #### Description
     * > Listens to any change to dimensions unit in the viewer. It will notify by calling
     * the provided parameter callback providing the new state everytime
     * the state changes.
     *
     * #### Usage
     *
     * ```typescript
     * visao.listenToDimensionsUnitChange((dimensionsUnitPayload: DimensionsUnitPayload) => {
     *   // Do Something
     * });
     * ```
     *
     * @param callback
     * Callback to execute when a change happen
     */
    Visao.prototype.listenToDimensionsUnitChange = function (callback) {
        this.callbacks.addTo(exports.ViewerAPIMessage.GET_DIMENSIONS_UNIT, callback);
    };
    /**
     * #### Description
     * > Removes a dimensions unit change listener.
     *
     * #### Usage
     *
     * ```typescript
     * visao.unListenToDimensionsUnitChange(callback);
     * ```
     *
     * @param callback
     * Callback which matches the one provided to a previously called {@link listenToDimensionsUnitChange}
     */
    Visao.prototype.unListenToDimensionsUnitChange = function (callback) {
        this.callbacks.removeByCallbackFunctionFrom(exports.ViewerAPIMessage.GET_DIMENSIONS_UNIT, callback);
    };
    /**
     * #### Description
     * > Listens to any change to the demos in the viewer. It will notify by calling
     * the provided parameter callback providing the new state everytime
     * the state changes.
     *
     * #### Usage
     *
     * ```typescript
     * visao.listenToDemosChange((demos: DemosPayload) => {
     *   // Do Something
     * });
     * ```
     *
     * @param callback
     * Callback to execute when a change happen
     */
    Visao.prototype.listenToDemosChange = function (callback) {
        this.callbacks.addTo(exports.ViewerAPIMessage.GET_DEMOS, callback);
    };
    /**
     * #### Description
     * > Removes a demos change listener.
     *
     * #### Usage
     *
     * ```typescript
     * visao.unListenToDemosChange(callback);
     * ```
     *
     * @param callback
     * Callback which matches the one provided to a previously called {@link listenToDemosChange}
     */
    Visao.prototype.unListenToDemosChange = function (callback) {
        this.callbacks.removeByCallbackFunctionFrom(exports.ViewerAPIMessage.GET_DEMOS, callback);
    };
    /**
     * #### Description
     * >  Get the data for all the demos and their steps.
     *
     * #### Usage
     *  > :warning: **{@link ViewerStatus}**  must be at **{@link ViewerStatus.LOADED}** to execute.
     *
     * ```typescript
     * // Get the demos data
     * let demos;
     * visao.getDemos((payload) => {
     *   demos = payload.demos;
     *   // Do something with the data
     * });
     * ```
     * @param callback
     * Callback with a payload as parameter which includes the list of demos.
     */
    Visao.prototype.getDemos = function (callback) {
        var callbackId = this.callbacks.addTo(exports.ViewerAPIMessage.GET_DEMOS, callback);
        this.executeAction({
            type: exports.ViewerAPIMessage.GET_DEMOS,
            callbackId: callbackId,
        }, exports.ViewerStatus.LOADED);
    };
    /**
     * #### Description
     * > Displays the provided demo step.
     *
     * #### Usage
     *  > :warning: **{@link ViewerStatus}**  must be at **{@link ViewerStatus.LOADED}** to execute.
     *
     * ```typescript
     * const stepName = 'shoe laces';
     * visao.showStep(stepName);
     * ```
     *
     * @param step
     * The name or the id of the step
     */
    Visao.prototype.showStep = function (step) {
        this.executeAction({
            type: exports.ViewerAPIMessage.SHOW_STEP,
            payload: {
                step: step,
            },
        }, exports.ViewerStatus.LOADED);
    };
    /**
     * #### Description
     * > Closes currently displayed demo step.
     *
     * #### Usage
     *  > :warning: **{@link ViewerStatus}**  must be at **{@link ViewerStatus.LOADED}** to execute.
     *
     * ```typescript
     * visao.closeStep();
     * ```
     */
    Visao.prototype.closeStep = function () {
        this.executeAction({
            type: exports.ViewerAPIMessage.CLOSE_STEP,
        }, exports.ViewerStatus.LOADED);
    };
    /**
     * #### Description
     * > Displays the previous demo step.
     *
     * #### Usage
     *  > :warning: **{@link ViewerStatus}**  must be at **{@link ViewerStatus.LOADED}** to execute.
     *
     * ```typescript
     * visao.previousStep();
     * ```
     */
    Visao.prototype.previousStep = function () {
        this.executeAction({
            type: exports.ViewerAPIMessage.PREVIOUS_STEP,
        }, exports.ViewerStatus.LOADED);
    };
    /**
     * #### Description
     * > Displays the next demo step.
     *
     * #### Usage
     *  > :warning: **{@link ViewerStatus}**  must be at **{@link ViewerStatus.LOADED}** to execute.
     *
     * ```typescript
     * visao.nextStep();
     * ```
     */
    Visao.prototype.nextStep = function () {
        this.executeAction({
            type: exports.ViewerAPIMessage.NEXT_STEP,
        }, exports.ViewerStatus.LOADED);
    };
    /**
     * #### Description
     * > Starts the demo steps player.
     *
     * #### Usage
     * > :warning: **{@link ViewerStatus}**  must be at **{@link ViewerStatus.LOADED}** to execute.
     *
     * ```typescript
     * visao.play();
     * ```
     */
    Visao.prototype.play = function () {
        this.executeAction({
            type: exports.ViewerAPIMessage.PLAY,
        }, exports.ViewerStatus.LOADED);
    };
    /**
     * #### Description
     * > Pauses the demo steps player.
     *
     * #### Usage
     * > :warning: **{@link ViewerStatus}**  must be at **{@link ViewerStatus.LOADED}** to execute.
     *
     * ```typescript
     * visao.pause();
     * ```
     */
    Visao.prototype.pause = function () {
        this.executeAction({
            type: exports.ViewerAPIMessage.PAUSE,
        }, exports.ViewerStatus.LOADED);
    };
    /**
     * #### Description
     * > Gets the current language information
     *
     * #### Usage
     * > :warning: **{@link ViewerStatus}**  must be either at **{@link ViewerStatus.MOUNTED}**,
     *  **{@link ViewerStatus.LOADING}**, **{@link ViewerStatus.LOADED}** to execute.
     *
     * ```typescript
     * visao.getLanguageInformation((info) => {
     *   const currentLanguage = info.language;
     *   const availableLanguages = info.languages;
     * });
     * ```
     *
     *  @param callback
     *  The callback will contain the currently displayed languages as well as all the supported languages in the viewer.
     */
    Visao.prototype.getLanguageInformation = function (callback) {
        var callbackId = this.callbacks.addTo(exports.ViewerAPIMessage.GET_LANGUAGE_INFO, callback);
        this.executeAction({
            type: exports.ViewerAPIMessage.GET_LANGUAGE_INFO,
            callbackId: callbackId,
        }, exports.ViewerStatus.MOUNTED);
    };
    /**
     * #### Description
     * > Tells the viewer to change the language of the different localized content if it can be found.
     * This will be visible on UI components like tooltips, menus, flyers, etc...
     *
     * #### Usage
     * > :warning: **{@link ViewerStatus}**  must be either at **{@link ViewerStatus.MOUNTED}**,
     *  **{@link ViewerStatus.LOADING}**, **{@link ViewerStatus.LOADED}** to execute.
     *
     * ```typescript
     * // Change the viewer language to french
     * visao.changeLanguage('fr');
     * ```
     *
     *  @param language
     *  The language abbreviation to apply. An exhaustive list of all valid languages
     *  can be found **[here](https://github.com/annexare/Countries/blob/master/data/languages.json)**.
     */
    Visao.prototype.changeLanguage = function (language) {
        this.executeAction({
            type: exports.ViewerAPIMessage.UPDATE_LANGUAGE,
            payload: {
                language: language,
            },
        }, exports.ViewerStatus.MOUNTED);
    };
    /**
     * #### Description
     * > Display one or more configuration/variant(s)
     * of the current 3D model if they can be found.
     *
     * #### Usage:
     * A 3D model of a shoe may support multiple independent variants,
     * such as color options for the shoe body, laces, and sole.
     * You can apply a single variant (e.g., just the body color),
     * or multiple variants at once (e.g., body, laces, and sole).
     *
     * > :warning: **{@link ViewerStatus}**  must be at **{@link ViewerStatus.LOADED}** to execute.
     *
     * ```typescript
     * // Change only the shoe body color to green
     * visao.showModelVariant('green');
     *
     * // Apply multiple variants: green body, white laces, and black sole
     * visao.showModelVariant(['green', 'white_laces', 'black_sole']);
     * ```
     *
     *  @param modelVariants
     *  A unique key or array of keys representing one or more variants that can be matched in the current 3D model schema.
     */
    Visao.prototype.showModelVariant = function (modelVariants) {
        this.executeAction({
            type: exports.ViewerAPIMessage.UPDATE_VARIANT,
            payload: {
                modelVariants: modelVariants,
            },
        }, exports.ViewerStatus.LOADED);
    };
    /**
     * #### Description
     * > Remove one or more applied configuration/variant(s)
     * from the current 3D model if they are currently active.
     *
     * #### Usage:
     * A 3D model of a shoe may have multiple variants applied simultaneously,
     * such as different colors for the body, laces, and sole.
     * You can remove a single variant, or several at once.
     *
     * > :warning: **{@link ViewerStatus}**  must be at **{@link ViewerStatus.LOADED}** to execute.
     *
     * ```typescript
     * // Remove the green body color variant
     * visao.removeModelVariant('green');
     *
     * // Remove multiple variants: white laces and black sole
     * visao.removeModelVariant(['white_laces', 'black_sole']);
     * ```
     *
     *  @param modelVariants
     *  A unique key that can be matched inside the current 3D model schema.
     */
    Visao.prototype.removeModelVariant = function (modelVariants) {
        this.executeAction({
            type: exports.ViewerAPIMessage.REMOVE_VARIANT,
            payload: {
                modelVariants: modelVariants,
            },
        }, exports.ViewerStatus.LOADED);
    };
    /**
     * #### Description
     * > Get the 3D model's variants/configurations information from the Visao viewer.
     * A configuration/variant is a structure with selected properties within the 3D model file,
     * which will determine how the 3D model will be displayed.
     *
     * #### Usage:
     * > :warning: **{@link ViewerStatus}**  must be at **{@link ViewerStatus.LOADED}** to execute.
     *
     * A 3D model of a shoe may have multiple variants applied simultaneously,
     * such as different colors for the body, laces, and sole.
     *
     * ```typescript
     * // Get the configurations/variants information
     * let variantsDisplayed;
     * let availableVariants;
     * visao.getVariantInformation((info) => {
     *   variantsDisplayed = info.selectedModelVariants;
     *   availableVariants = info.availableModelVariants;
     *   // Do something with the data
     * });
     * ```
     *
     *  @param callback
     *  The configuration/variant information will be the parameter of the provided callback once called.
     */
    Visao.prototype.getVariantInformation = function (callback) {
        var callbackId = this.callbacks.addTo(exports.ViewerAPIMessage.GET_VARIANT_INFO, callback);
        this.executeAction({
            type: exports.ViewerAPIMessage.GET_VARIANT_INFO,
            callbackId: callbackId,
        }, exports.ViewerStatus.LOADED);
    };
    /**
     * #### Description
     * > Get the 3D model's animation information from the Visao viewer.
     *
     * #### Usage:
     * > :warning: **{@link ViewerStatus}**  must be at **{@link ViewerStatus.LOADED}** to execute.
     *
     * ```typescript
     * // Get the animation information
     * let availableAnimations;
     * let currentAnimation;
     * visao.getAnimationInformation((info) => {
     *   availableAnimations = info.animations;
     *   currentAnimation = info.animation;
     *   // Do something with the data
     * });
     * ```
     *
     *  @param callback
     *  The animation information will be the parameter of the provided callback once called.
     */
    Visao.prototype.getAnimationInformation = function (callback) {
        var callbackId = this.callbacks.addTo(exports.ViewerAPIMessage.GET_ANIMATION_INFO, callback);
        this.executeAction({
            type: exports.ViewerAPIMessage.GET_ANIMATION_INFO,
            callbackId: callbackId,
        }, exports.ViewerStatus.LOADED);
    };
    /**
     * #### Description
     * > Start the passed animation.
     *
     * #### Usage:
     * Default or step animations will be overridden by the new animation.
     *
     * > :warning: **{@link ViewerStatus}**  must be at **{@link ViewerStatus.LOADED}** to execute.
     *
     * ```typescript
     * // Start the animation
     * visao.startAnimation(animation);
     * ```
     * ```typescript
     * // Start the animation with options
     * visaoAPI.startAnimation(anim, {
     *         repetitionType: AnimationRepetitionType.REPEAT,
     *         repetitionQuantity: 2,
     *       })
     * ```
     *
     *  @param animation
     *  The animation name to start
     *
     *
     *  @param options
     *  The animation options
     */
    Visao.prototype.startAnimation = function (animation, options) {
        if (options === void 0) { options = {}; }
        this.executeAction({
            type: exports.ViewerAPIMessage.START_ANIMATION,
            payload: {
                animation: animation,
                options: options,
            },
        }, exports.ViewerStatus.LOADED);
    };
    /**
     * #### Description
     * > Stop the current animation.
     *
     * #### Usage:
     * Viewer preset animations (default or step) will be restored. This can be prevented by parameter preventPresets being set to true.
     *
     * > :warning: **{@link ViewerStatus}**  must be at **{@link ViewerStatus.LOADED}** to execute.
     *
     * ```typescript
     * // Stop the animation (preset animation will be restored if available)
     * visao.stopAnimation();
     * ```
     *
     * ```typescript
     * // Stop the animation (preset animation will not be restored)
     * visao.stopAnimation(true);
     * ```
     *
     *  @param preventPresets
     *  Whether preset (default & step) animations should be prevented from being restored, Default: false
     */
    Visao.prototype.stopAnimation = function (preventPresets) {
        if (preventPresets === void 0) { preventPresets = false; }
        this.executeAction({
            type: exports.ViewerAPIMessage.STOP_ANIMATION,
            payload: {
                preventPresets: preventPresets,
            },
        }, exports.ViewerStatus.LOADED);
    };
    /**
     * #### Description
     * > Sets the dimensions unit.
     *
     * #### Usage
     *  > :warning: **{@link ViewerStatus}**  must be at **{@link ViewerStatus.LOADED}** to execute.
     *
     * ```typescript
     * const unit = 'm';
     * visao.setDimensionsUnit(unit);
     * ```
     *
     * @param unit
     * The new unit.
     */
    Visao.prototype.setDimensionsUnit = function (unit) {
        this.executeAction({
            type: exports.ViewerAPIMessage.UPDATE_DIMENSIONS_UNIT,
            payload: {
                unit: unit,
            },
        }, exports.ViewerStatus.LOADED);
    };
    /**
     * #### Description
     * > Starts the augmented reality feature.
     *
     * #### Usage
     * > :warning: **{@link ViewerStatus}**  must be at **{@link ViewerStatus.LOADED}** to execute. This feature only
     *  works on devices supporting augmented reality.
     *
     * ```typescript
     * visao.startAR();
     * ```
     */
    Visao.prototype.startAR = function () {
        this.executeAction({
            type: exports.ViewerAPIMessage.START_AR,
        }, exports.ViewerStatus.LOADED);
    };
    /**
     * #### Description
     * > Reset the viewer's camera to its original state.
     *
     * #### Usage
     *  > :warning: **{@link ViewerStatus}**  must be at **{@link ViewerStatus.LOADED}** to execute.
     *
     * ```typescript
     * visao.resetCamera();
     * ```
     */
    Visao.prototype.resetCamera = function () {
        this.executeAction({
            type: exports.ViewerAPIMessage.RESET_CAMERA,
        }, exports.ViewerStatus.LOADED);
    };
    /**
     * #### Description
     * > Captures a screenshot of the viewer's 3D model.
     * This will trigger a download of the image.
     *
     * #### Usage
     *  > :warning: **{@link ViewerStatus}**  must be at **{@link ViewerStatus.LOADED}** to execute.
     *
     * ```typescript
     * visao.captureScreenshot({
     *  mimeType: 'image/png',
     * });
     * ```
     * @param options
     * {@link ScreenshotOptions}
     * Optional screenshot options which can include the `mimeType` and `fileFormat`.
     * The default values are `mimeType: 'image/png'` and `fileFormat: 'blob'`.
     */
    Visao.prototype.captureScreenshot = function (options) {
        if (options === void 0) { options = {}; }
        this.executeAction({
            type: exports.ViewerAPIMessage.CAPTURE_SCREENSHOT,
            payload: __assign(__assign({}, defaultScreenshotOptions), options),
        }, exports.ViewerStatus.LOADED);
    };
    /**
     * #### Description
     * >  Captures a screenshot of the viewer's 3D model and then returns the image blob.
     *
     * #### Usage
     *  > :warning: **{@link ViewerStatus}**  must be at **{@link ViewerStatus.LOADED}** to execute.
     *
     * ```typescript
     * // Get the screenshot blob
     * visao.getScreenshot((screenshot: Blob | string) => {
     *   // Do something with the screenshot
     * }, {
     *   mimeType: 'image/png',
     *   fileFormat: 'blob',
     * });
     * ```
     * @param callback
     * Callback which includes the captured screenshot blob as parameter.
     * @param options
     * {@link ScreenshotOptions}
     * Optional screenshot options which can include the `mimeType` and `fileFormat`.
     * The default values are `mimeType: 'image/png'` and `fileFormat: 'blob'`.
     */
    Visao.prototype.getScreenshot = function (callback, options) {
        if (options === void 0) { options = {}; }
        var callbackId = this.callbacks.addTo(exports.ViewerAPIMessage.GET_SCREENSHOT, callback);
        this.executeAction({
            type: exports.ViewerAPIMessage.GET_SCREENSHOT,
            callbackId: callbackId,
            payload: __assign(__assign({}, defaultScreenshotOptions), options),
        }, exports.ViewerStatus.LOADED);
    };
    /**
     * #### Description
     * > Locks the viewer's camera controls.
     *
     * #### Usage
     *  > :warning: **{@link ViewerStatus}**  must be at **{@link ViewerStatus.LOADED}** to execute.
     *
     * ```typescript
     * visao.lockCamera();
     * ```
     */
    Visao.prototype.lockCamera = function () {
        this.executeAction({
            type: exports.ViewerAPIMessage.LOCK_CAMERA,
        }, exports.ViewerStatus.LOADED);
    };
    /**
     * #### Description
     * > Unlocks the viewer's camera controls.
     *
     * #### Usage
     *  > :warning: **{@link ViewerStatus}**  must be either at **{@link ViewerStatus.MOUNTED}**,
     *  **{@link ViewerStatus.LOADING}**, **{@link ViewerStatus.LOADED}** to execute.
     *
     * ```typescript
     * visao.unlockCamera();
     * ```
     */
    Visao.prototype.unlockCamera = function () {
        this.executeAction({
            type: exports.ViewerAPIMessage.UNLOCK_CAMERA,
        }, exports.ViewerStatus.MOUNTED);
    };
    /**
     * #### Description
     * > Shows the viewer's help panel.
     *
     * #### Usage
     *  > :warning: **{@link ViewerStatus}**  must be either at **{@link ViewerStatus.MOUNTED}**,
     *  **{@link ViewerStatus.LOADING}**, **{@link ViewerStatus.LOADED}** to execute.
     *
     * ```typescript
     * visao.showHelp();
     * ```
     */
    Visao.prototype.showHelp = function () {
        this.executeAction({
            type: exports.ViewerAPIMessage.SHOW_HELP,
        }, exports.ViewerStatus.MOUNTED);
    };
    /**
     * #### Description
     * > Closes the viewer's help panel.
     *
     * #### Usage
     *  > :warning: **{@link ViewerStatus}**  must be either at **{@link ViewerStatus.MOUNTED}**,
     *  **{@link ViewerStatus.LOADING}**, **{@link ViewerStatus.LOADED}** to execute.
     *
     * ```typescript
     * visao.closeHelp();
     * ```
     */
    Visao.prototype.closeHelp = function () {
        this.executeAction({
            type: exports.ViewerAPIMessage.CLOSE_HELP,
        }, exports.ViewerStatus.MOUNTED);
    };
    /**
     * #### Description
     * > Shows the viewer's QR code panel.
     *
     * #### Usage
     *  > :warning: **{@link ViewerStatus}**  must be either at **{@link ViewerStatus.MOUNTED}**,
     *  **{@link ViewerStatus.LOADING}**, **{@link ViewerStatus.LOADED}** to execute.
     *
     * ```typescript
     * visao.showQR();
     * ```
     */
    Visao.prototype.showQR = function () {
        this.executeAction({
            type: exports.ViewerAPIMessage.SHOW_QR,
        }, exports.ViewerStatus.MOUNTED);
    };
    /**
     * #### Description
     * > Closes the viewer's QR code panel.
     *
     * #### Usage
     *  > :warning: **{@link ViewerStatus}**  must be either at **{@link ViewerStatus.MOUNTED}**,
     *  **{@link ViewerStatus.LOADING}**, **{@link ViewerStatus.LOADED}** to execute.
     *
     * ```typescript
     * visao.closeQR();
     * ```
     */
    Visao.prototype.closeQR = function () {
        this.executeAction({
            type: exports.ViewerAPIMessage.CLOSE_QR,
        }, exports.ViewerStatus.MOUNTED);
    };
    /**
     * #### Description
     * > Shows the viewer's dimensions.
     *
     * #### Usage
     * > :warning: **{@link ViewerStatus}**  must be at **{@link ViewerStatus.LOADED}** to execute.
     *
     * ```typescript
     * visao.showDimensions();
     * ```
     */
    Visao.prototype.showDimensions = function () {
        this.executeAction({
            type: exports.ViewerAPIMessage.SHOW_DIMENSIONS,
        }, exports.ViewerStatus.LOADED);
    };
    /**
     * #### Description
     * > Closes the viewer's dimensions.
     *
     * #### Usage
     * > :warning: **{@link ViewerStatus}**  must be at **{@link ViewerStatus.LOADED}** to execute.
     *
     * ```typescript
     * visao.closeDimensions();
     * ```
     */
    Visao.prototype.closeDimensions = function () {
        this.executeAction({
            type: exports.ViewerAPIMessage.CLOSE_DIMENSIONS,
        }, exports.ViewerStatus.LOADED);
    };
    /**
     * #### Description
     * > Starts the viewer's auto-rotation.
     *
     * #### Usage
     * > :warning: **{@link ViewerStatus}**  must be at **{@link ViewerStatus.LOADED}** to execute.
     *
     * ```typescript
     * visao.startAutoRotate();
     * ```
     */
    Visao.prototype.startAutoRotate = function () {
        this.executeAction({
            type: exports.ViewerAPIMessage.START_AUTO_ROTATE,
        }, exports.ViewerStatus.LOADED);
    };
    /**
     * #### Description
     * > Stops the viewer's auto-rotation.
     *
     * #### Usage
     * > :warning: **{@link ViewerStatus}**  must be at **{@link ViewerStatus.LOADED}** to execute.
     *
     * ```typescript
     * visao.stopAutoRotate();
     * ```
     */
    Visao.prototype.stopAutoRotate = function () {
        this.executeAction({
            type: exports.ViewerAPIMessage.STOP_AUTO_ROTATE,
        }, exports.ViewerStatus.LOADED);
    };
    /**
     * #### Description
     * > Mutes the viewer's voiceover.
     *
     * #### Usage
     * > :warning: **{@link ViewerStatus}**  must be at **{@link ViewerStatus.MOUNTED}** to execute.
     *
     * ```typescript
     * visao.muteVoiceover();
     * ```
     */
    Visao.prototype.muteVoiceover = function () {
        this.executeAction({
            type: exports.ViewerAPIMessage.MUTE_VOICEOVER,
        }, exports.ViewerStatus.MOUNTED);
    };
    /**
     * #### Description
     * > Un-mutes the viewer's voiceover.
     *
     * #### Usage
     * > :warning: **{@link ViewerStatus}**  must be at **{@link ViewerStatus.MOUNTED}** to execute.
     *
     * ```typescript
     * visao.unmuteVoiceover();
     * ```
     */
    Visao.prototype.unmuteVoiceover = function () {
        this.executeAction({
            type: exports.ViewerAPIMessage.UNMUTE_VOICEOVER,
        }, exports.ViewerStatus.MOUNTED);
    };
    Visao.prototype.executeAction = function (action, statusNeeded) {
        var _a, _b;
        this.logInvalidViewerElement();
        this.logForInsufficientStatusLevel(statusNeeded);
        var message = __assign(__assign({}, action), { source: exports.ViewerAPIMessageSource.API });
        (_b = (_a = this.viewerElement) === null || _a === void 0 ? void 0 : _a.contentWindow) === null || _b === void 0 ? void 0 : _b.postMessage(JSON.stringify(message), '*');
    };
    Visao.prototype.logInvalidViewerElement = function () {
        if (!this.viewerElement) {
            logErrorBlock([
                "Viewer HTML element cannot be found in the DOM with the id \"".concat(this.id, "\"."),
                'Make sure the visao iframe is properly mounted in the DOM before calling this method.',
            ]);
        }
    };
    Visao.prototype.logForInsufficientStatusLevel = function (statusNeeded) {
        if (statusNeeded &&
            !this.validateStatusHasReachedNeededLevel(statusNeeded)) {
            logErrorBlock([
                'The viewer is not ready for this action.',
                "Minimum viewer status needed: [".concat(statusNeeded, "]"),
                "Current viewer status: [".concat(this.status, "]"),
            ]);
        }
    };
    Visao.prototype.validateStatusHasReachedNeededLevel = function (statusNeeded) {
        return statusLevels[this.status] >= statusLevels[statusNeeded];
    };
    return Visao;
}());

exports.Demo = Demo;
exports.LocalizedContent = LocalizedContent;
exports.Step = Step;
exports.Visao = Visao;
exports.defaultScreenshotOptions = defaultScreenshotOptions;
exports.listenToMessages = listenToMessages;
exports.parseIncomingMessage = parseIncomingMessage;
exports.statusLevels = statusLevels;
exports.unListenToMessages = unListenToMessages;
//# sourceMappingURL=index.cjs.map
